
import os
import socket
import subprocess
import sys
import glob

from shutil import copyfile

from charmhelpers.core.host import lsb_release
from charmhelpers.core import hookenv, host
from charmhelpers.fetch import (
    apt_update,
    add_source,
    apt_install
)

zbx_repo_list = '/etc/apt/sources.list.d/zabbix.list'
apt_key_file = 'keys/repo.zabbix.com'

zbx_repo_url = 'http://repo.zabbix.com/zabbix/2.4/ubuntu'
ubuntu_sources_url = 'http://archive.ubuntu.com/ubuntu'

TEMPLATES = 'templates'


def juju_log(severity, message):
    hookenv.log(message, severity)


def get_config_path(service_name):
    """ Get config name and path by service name """

    if intern(service_name) is intern('zabbix-server'):
        return '/etc/zabbix/zabbix_server.conf'
    if intern(service_name) is intern('zabbix-proxy'):
        return '/etc/zabbix/zabbix_proxy.conf'
    if intern(service_name) is intern('zabbix-agent'):
        return '/etc/zabbix/zabbix_agentd.conf'
    if intern(service_name) is intern('zabbix-java-gateway'):
        return '/etc/zabbix/zabbix_java_gateway.conf'
    if intern(service_name) is intern('zabbix-frontend'):
        return '/etc/zabbix/web/zabbix.conf.php'
    if intern(service_name) is intern('zabbix-apache'):
        return '/etc/apache2/conf-enabled/zabbix.conf'

    return None


def add_sources(service_name=None):
    """ Add repositories by service name """
    run("apt-key add {}".format(apt_key_file))

    sources = open(zbx_repo_list, 'w')

    os_release = get_os_release()

    sources.write('deb {} {} main\n'.format(zbx_repo_url, os_release))
    sources.write('deb-src {} {} main\n'.format(zbx_repo_url, os_release))

    if service_name in ['zabbix-server', 'zabbix-proxy']:
        add_source('deb {} {} main multiverse'.format(ubuntu_sources_url,
                                                      os_release))

    sources.close()

    apt_update()


def pre_install():
    """ Prepare installation """
    if os.path.isdir('exec.d'):
        for f in glob.glob("exec.d/*/charm-pre-install"):
            if not os.access(f, os.X_OK):
                continue
            try:
                subprocess.check_output([f])
            except subprocess.CalledProcessError as exc:
                juju_log('ERROR', str(exc))
                sys.exit(1)

        apt_update()


def install_packages(packages=[], fatal=True):
    """ Install specified packages """
    options = ['--option=Dpkg::Options::=--force-confold',
               '--no-install-suggests',
               '--no-install-recommends']

    if packages.__len__() == 0:
        return False

    return apt_install(packages, options, fatal)


def check_config_perm(config_file=None, Read=True):
    """ Check configuration file permissions and availability """
    permissions = os.R_OK

    if config_file is None:
        return False

    if Read is False:
        permissions = os.W_OK

    if (not os.path.exists(config_file) or
       not os.access(config_file, permissions)):
        return False
    else:
        return True


def copy_config(config_file=None):
    """ Copy config to specified folder """
    filename = os.path.basename(config_file)

    copyfile(TEMPLATES + '/' + filename, config_file)


def get_relations(name=None, cnt=False):
    """ Getting relations by name """
    relations = hookenv.relations()

    result = {}
    cnt_output = 0

    if name is not None:
        for rel_type in relations:
            if name is not None and intern(rel_type) is intern(name):
                if relations[rel_type].__len__() > 0:
                    result = {rel_type: relations[rel_type]}
    else:
        result = relations

    if cnt is True:
        for rel_type in result:
            for rel_name in result[rel_type]:
                cnt_output += result[rel_type][rel_name].__len__() - 1
        return cnt_output

    return result


def get_db_relations(name=None, cnt=False):
    """ List of database relations for all known database types """
    db_types = ['database-mysql', 'database-postgresql', 'database-oracle', 'database-db2', 'database-sqlite3']
    relations = get_relations(name, False)

    databases = {}
    cnt_output = 0

    for rel_name in relations:
        rel_name_tmp = rel_name.rsplit(':', 1)[0]
        if rel_name_tmp in db_types:
            if name is None and relations[rel_name].__len__() > 0:
                databases = {rel_name: relations[rel_name]}
            elif name is not None and intern(name) is intern(rel_name_tmp):
                databases = {rel_name: relations[rel_name]}

    if cnt is True:
        for rel_type in databases:
            for rel_name in databases[rel_type]:
                cnt_output += databases[rel_type][rel_name].__len__() - 1
        return cnt_output

    return databases


def get_proxy_relations(cnt=False):
    """ Getting all proxy host names """
    proxy_types = ['proxy-passive', 'proxy-active']
    relations = get_relations(None, False)

    proxies = []

    for rel_type in relations:
        if rel_type not in proxy_types:
            continue
        for rel_name in relations[rel_type]:
            for rel in relations[rel_type][rel_name]:
                if 'proxy' not in rel:
                    continue
                relation = relations[rel_type][rel_name][rel]
                try:
                    relation['hostname']
                except:
                    pass
                else:
                    proxies.append(relation['hostname'])

    if cnt is True:
        return proxies.__len__()

    return proxies


def port_manage(port_num=None, action='open'):
    """ Manage TCP port """
    if action is 'open':
        hookenv.open_port(port_num)
    else:
        hookenv.close_port(port_num)


def get_unit_hostname():
    """ Host name """
    return socket.gethostname()


def get_os_release():
    """ Distribution release """
    return lsb_release()['DISTRIB_CODENAME']


def run(command, exit_on_error=True, quiet=False):
    """ Run a command and return the output """
    if not quiet:
        juju_log('DEBUG', "Running {!r}".format(command))
    p = subprocess.Popen(
        command, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
        shell=isinstance(command, basestring))
    p.stdin.close()
    lines = []
    for line in p.stdout:
        if line:
            # LP:1274460 & LP:1259490 mean juju-log is no where near as
            # useful as we would like, so just shove a copy of the
            # output to stdout for logging.
            # log("> {}".format(line), DEBUG)
            if not quiet:
                print line
            lines.append(line)
        elif p.poll() is not None:
            break

    p.wait()

    if p.returncode == 0:
        return '\n'.join(lines)

    if p.returncode != 0 and exit_on_error:
        juju_log('ERROR', "ERROR: {}".format(p.returncode))
        sys.exit(p.returncode)

    raise subprocess.CalledProcessError(
        p.returncode, command, '\n'.join(lines))


def service_available(service_name):
    """Determine whether a system service is available"""
    try:
        subprocess.check_output(['service', service_name, 'status'],
                                stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        if e.returncode not in [0, 3]:
            return False
        else:
            return True
    else:
        return True


def service_restart(service_name):
    """ Soft restart servise """
    result = True

    if service_available(service_name) is True:
        if host.service_running(service_name) is True:
            juju_log('DEBUG', '{} restarting...'.format(service_name))
            result = host.service_restart(service_name)
        else:
            juju_log('DEBUG', '{} starting...'.format(service_name))
            result = host.service_start(service_name)

        if result is False:
            juju_log('ERROR', 'Could not restart {}'.format(service_name))
    else:
        juju_log('ERROR', 'Service {} is not available to restart'.format(service_name))
        result = False

    return result


def update_relations(rel_name, relation_settings=None, **kwargs):
    """ Mass update relation variables by relation name """
    options = {}

    for k, v in (kwargs.items()):
        if v is None:
            options[k] = ''
        else:
            options[k] = v

    for relid in hookenv.relation_ids(rel_name):
        hookenv.relation_set(relid, options)
